<?php
App::uses('EvShopAppModel', 'EvShop.Model');
App::uses('ArrayUtil', 'EvCore.Lib');

/**
 * Variant Model
 *
 * @property Product $Product
 * @property Option $Option
 */
class Variant extends EvShopAppModel {

	/**
	 * Display field
	 *
	 * @var string
	 */
	public $displayField = 'name';

	/**
	 * Validation rules
	 *
	 * @var array
	 */
	public $validate = array(
		'product_id' => array(
			'numeric' => array(
				'rule' => array('numeric')
			),
		),
		'name' => array(
			'notEmpty' => array(
				'rule' => array('notEmpty'),
				'message' => 'A variant name must be entered.'
			),
		),
		'price' => array(
			'notEmpty' => array(
				'rule' => array('notEmpty')
			),
			'money' => array(
				'rule' => array('money')
			),
		),
	);

	//The Associations below have been created with all possible keys, those that are not needed can be removed

	/**
	 * belongsTo associations
	 *
	 * @var array
	 */
	public $belongsTo = array(
		'Product' => array(
			'className' => 'EvShop.Product'
		)
	);

	/**
	 * hasAndBelongsToMany associations
	 *
	 * @var array
	 */
	public $hasAndBelongsToMany = array(
		'Option' => array(
			'className' => 'EvShop.Option',
			'joinTable' => 'ev_shop_options_variants'
		)
	);

	/**
	 * take the multiple option arrays and build variants
	 *
	 * @param 	array 	array of option ids
	 * @param 	string 	The entered product name as a fallback if no options were selected
	 * @return 	array 	array of variants ready for DB insertion
	 */
	public function buildVariants($options, $productName) {
		if (empty($options) || ! is_array($options)) {
			return $this->defaultInsertArray($productName);
		}

		$Option = EvClassRegistry::init('EvShop.Option');
		$optionNames = $Option->getOptionNames(
			Hash::flatten($options)
		);

		$cartesian = ArrayUtil::cartesianProduct($options);

		return $this->generateInsertArray(
			$cartesian,
			$optionNames
		);
	}

	/**
	 * from the cartesian array generate db insert array
	 *
	 * @param 	array 	Cartesian product arrays
	 * @return 	array 	Array with variant name and option links
	 */
	public function generateInsertArray($cartesianProduct, $optionNames) {
		$variants = array();

		foreach ($cartesianProduct as $optionIds) {
			$name = array_intersect_key(
				$optionNames,
				array_flip($optionIds)
			);

			$variants[] = array(
				'name' => implode(' ', $name),
				'is_active' => 1,
				'Option' => array(
					'Option' => $optionIds
				)
			);
		}

		return $variants;
	}

	/**
	 * generate a default Insert Array for a product with no options
	 * variant = product name
	 *
	 * @param 	string 	Product name
	 * @return 	array 	Array with variant data for insert
	 */
	public function defaultInsertArray($productName) {
		return array(
			array(
				'name' => $productName,
				'is_active' => 1,
			)
		);
	}
}
