<?php
App::uses('Variant', 'Model');

/**
 * Variant Test Case
 *
 */
class VariantTest extends CakeTestCase {

/**
 * Fixtures
 *
 * @var array
 */
	public $fixtures = array(
		'plugin.EvShop.variant',
		'plugin.EvShop.product',
		'plugin.EvShop.brand',
		//'plugin.EvShop.tax_level',
		'plugin.EvShop.category',
		//'plugin.EvShop.categories_product',
		'plugin.EvShop.option',
		'plugin.EvShop.option_group',
		//'plugin.EvShop.options_variant'
	);

/**
 * setUp method
 *
 * @return void
 */
	public function setUp() {
		parent::setUp();
		$this->Variant = ClassRegistry::init('EvShop.Variant');
	}

/**
 * tearDown method
 *
 * @return void
 */
	public function tearDown() {
		unset($this->Variant);

		parent::tearDown();
	}

	public function testdefaultInsertArrayGeneratesCorrectArrayForDB() {
		$this->assertEquals(
			$this->Variant->defaultInsertArray('My Product'),
			array(
				array(
					'name' => 'My Product',
					'is_active' => 1
				)
			)
		);
	}

	public function testgenerateInsertArrayReturnsValidArrayForDB() {
		$this->assertEquals(
			$this->Variant->generateInsertArray(
				array(
					array(1, 4),
					array(1, 5),
					array(2, 4),
					array(2, 5)
				),
				array(
					'1' => 'Red',
					'2' => 'Blue',
					'4' => 'Matte',
					'5' => 'Gloss'
				)
			),
			array(
				array(
					'name' => 'Red Matte',
					'is_active' => 1,
					'Option' => array(
						'Option' => array(
							1, 4
						)
					)
				),
				array(
					'name' => 'Red Gloss',
					'is_active' => 1,
					'Option' => array(
						'Option' => array(
							1, 5
						)
					)
				),
				array(
					'name' => 'Blue Matte',
					'is_active' => 1,
					'Option' => array(
						'Option' => array(
							2, 4
						)
					)
				),
				array(
					'name' => 'Blue Gloss',
					'is_active' => 1,
					'Option' => array(
						'Option' => array(
							2, 5
						)
					)
				)
			)
		);
	}

	public function testBuildVariantsDefaultsToProductNameWithNoOptions() {
		$this->assertEquals(
			$this->Variant->buildVariants(
				array(),
				'My Product'
			),
			array(
				array(
					'name' => 'My Product',
					'is_active' => 1
				)
			)
		);

		$this->assertEquals(
			$this->Variant->buildVariants(
				null,
				'My Product'
			),
			array(
				array(
					'name' => 'My Product',
					'is_active' => 1
				)
			)
		);

		$this->assertEquals(
			$this->Variant->buildVariants(
				false,
				'My Product'
			),
			array(
				array(
					'name' => 'My Product',
					'is_active' => 1
				)
			)
		);
	}

	public function testBuildVariantsReturnsCorrectArrayWithOptions() {
		$this->assertEquals(
			$this->Variant->buildVariants(
				array(
					2 => array(
						1, 2
					),
					3 => array(
						4, 5
					)
				),
				'My Product'
			),
			array(
				array(
					'name' => 'Red Matte',
					'is_active' => 1,
					'Option' => array(
						'Option' => array(
							1, 4
						)
					)
				),
				array(
					'name' => 'Red Gloss',
					'is_active' => 1,
					'Option' => array(
						'Option' => array(
							1, 5
						)
					)
				),
				array(
					'name' => 'Blue Matte',
					'is_active' => 1,
					'Option' => array(
						'Option' => array(
							2, 4
						)
					)
				),
				array(
					'name' => 'Blue Gloss',
					'is_active' => 1,
					'Option' => array(
						'Option' => array(
							2, 5
						)
					)
				)
			)
		);
	}
}
